"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ulid_1 = require("ulid");
const ws_1 = require("ws");
const __1 = require("../");
const schema_1 = __importDefault(require("./schema"));
const createMemoryEventProcessor_1 = __importDefault(require("../createMemoryEventProcessor"));
const MemoryEventStore_1 = __importDefault(require("../MemoryEventStore"));
const MemorySubscriptionManager_1 = __importDefault(require("../MemorySubscriptionManager"));
const WebSocketConnectionManager_1 = __importDefault(require("../WebSocketConnectionManager"));
class TestLambdaServer {
    constructor({ port = 3001 } = {}) {
        this.close = async () => {
            const wsClose = new Promise((resolve, reject) => {
                this.wsServer.close(err => {
                    clearInterval(this.eventProcessingInterval);
                    return err ? reject(err) : resolve();
                });
            });
            return wsClose;
        };
        this.processEvents = () => {
            const events = this.eventStore.events.splice(0, 5);
            if (events) {
                this.eventProcessor(events);
            }
        };
        this.start = async () => {
            const wsStart = new Promise((resolve, reject) => {
                this.wsServer = new ws_1.Server({ port: this.port });
                this.wsServer.on('connection', async (ws) => {
                    const connectionId = ulid_1.ulid();
                    const result = await this.handler({
                        requestContext: {
                            connectionId,
                            routeKey: '$connect',
                            socket: ws,
                        },
                    }, {});
                    ws.on('close', () => {
                        this.handler({
                            requestContext: {
                                connectionId,
                                routeKey: '$disconnect',
                                socket: ws,
                            },
                        }, {});
                    });
                    ws.on('message', async (data) => {
                        const result = await this.handler({
                            requestContext: {
                                connectionId,
                                routeKey: '$default',
                                socket: ws,
                            },
                            body: data.toString('utf8'),
                        }, {});
                        if (result) {
                            ws.send(result.body);
                        }
                    });
                    if (result && result.statusCode === 403) {
                        ws.send(result.body);
                        ws.close();
                        return;
                    }
                    if (result) {
                        const conn = this.connectionManager.connections.get(connectionId);
                        if (conn) {
                            conn.socket = ws;
                        }
                        ws.send(result.body);
                    }
                });
                this.wsServer.on('listening', () => {
                    // start event processing
                    this.eventProcessingInterval = setInterval(this.processEvents, 20);
                    this.wsServer.removeListener('error', reject);
                    resolve();
                });
                this.wsServer.on('error', reject);
            });
            return wsStart;
        };
        this.eventStore = new MemoryEventStore_1.default();
        this.port = port;
        const schema = schema_1.default({
            pubSub: new __1.PubSub({ eventStore: this.eventStore }),
        });
        this.connectionManager = new WebSocketConnectionManager_1.default();
        this.subscriptionManager = new MemorySubscriptionManager_1.default();
        this.handler = __1.createWsHandler({
            schema,
            connectionManager: this.connectionManager,
            subscriptionManager: this.subscriptionManager,
        });
        this.eventProcessor = createMemoryEventProcessor_1.default({
            schema,
            connectionManager: this.connectionManager,
            subscriptionManager: this.subscriptionManager,
        });
    }
}
exports.TestLambdaServer = TestLambdaServer;
